class Session < ApplicationRecord
  belongs_to :conference
  belongs_to :stage
  has_many :assignments
  has_many :users, through: :assignments

  scope :scheduled, -> { where(status: 'scheduled') }

  validates :ref_id, uniqueness: { scope: :conference_id }

  after_update :notify_if_changed

  self.implicit_order_column = :starts_at

  def to_param
    ref_id
  end

  def starts_at
    super.in_time_zone(conference.time_zone)
  end

  def ends_at
    super.in_time_zone(conference.time_zone)
  end

  def is_interpreted
    # XXX
    ["Stage 1", "Stage 2"].include? stage.name
  end

  def translators_needed?
    is_interpreted && assignments.length < 2
  end

  def backup_needed?
    is_interpreted && assignments.length < 4
  end

  def has_assignees?
    assignments.length > 0
  end

  private

  def notify_if_changed
    return if new_record?
    if saved_changes.present?
      ActiveSupport::Notifications.instrument("session.updated", record: self, changes: saved_changes)
    end
  end
end
